# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shutil
import sys
import unittest

from errno import EEXIST

import gvn.cmdline
import gvn.errors
import testing.testcommon
import testing.test_wc


class TestCase(testing.test_wc.TestCase):
  def setUp(self):
    testing.test_wc.TestCase.setUp(self)

    # self.wcdir is already internal style (as it should be, for all
    # the other tests using it), but in order to test the Context
    # class's automatic internalizing, we need both local and internal.
    self.wcdir_internal_style = self.wcdir
    if sys.platform == 'win32':
      # internal == local for unix; set local for win32.
      (drive, path) = os.path.splitdrive(self.wcdir_internal_style)
      self.wcdir = os.path.join(drive, '\\',
                                os.path.join(*path.strip('/').split('/')))

    self.Ctx()

  def Ctx(self):
    (options, argv) = gvn.cmdline.OptionParser().parse_args([])
    self.ctx = gvn.cmdline.Context(options, argv)
    self.ctx.config = self.config
    return self.ctx


class TestCmdline(TestCase):
  def testRunSvnCommand(self):
    self.assertRaises(gvn.errors.BadCommand,
                      gvn.cmdline.RunSvnCommand, self.ctx, 'bork', [],
                      runner=lambda argv: 0)
    result = []
    gvn.cmdline.RunSvnCommand(self.ctx, 'ls', ['//'],
                              runner=lambda argv: result.extend(argv))
    cfgdir = os.path.realpath(os.path.join(os.getenv('HOME'), '.subversion'))

    self.assertEquals(result, ['ls', '--config-dir', cfgdir,
                               '--', self.repo_url])


class TestContext(TestCase):
  def setUp(self):
    TestCase.setUp(self)

    self.projects_dir = self.config.project_dirs[0]
    try:
      os.makedirs(self.projects_dir)
    except OSError, e:
      if e.errno != EEXIST:
        raise

    # Make sure neither of these URLs is the same as the URL for the
    # default test in testProject.

    # Create project file with URL of repository root.
    fp = open(os.path.join(self.projects_dir, 'root'), 'w')
    fp.write('[project]\n'
             'url = %s\n' % (self.repo_url,))
    fp.close()

    # Create project file with URL of public/projects.
    fp = open(os.path.join(self.projects_dir, 'pub'), 'w')
    fp.write('[project]\n'
             'url = %s\n' % (self.repo_url + '/public/projects',))
    fp.close()

  def testNoWc(self):
    # Move up out of the wc.
    os.chdir('..')

    self.assertRaises(gvn.errors.NotWC, self.ctx.DisplayPath, '')
    self.assertRaises(gvn.errors.NotWC, lambda: self.ctx.wc)
    self.assertRaises(gvn.errors.NoProject, lambda: self.ctx.project)

  def testNonWcOperands(self):
    self.ctx._operands = [os.path.join('a', 'b')]
    self.ctx.wc_operands = False

    # Note that this only tests something on Windows; on unix it's /
    # no matter what.
    self.assertEquals(self.ctx.operands, [os.path.join('a', 'b')])

    self.assertEquals(self.ctx.path, '')
    self.assertEquals(self.ctx.DisplayPath('greek-tree/A'),
                      os.path.join('greek-tree', 'A'))
    self.assertEquals(self.ctx.wc.path, self.wcdir_internal_style)
    self.assertEquals(self.ctx.project.path, 'public/projects')

  def testWcOperandsMixed(self):
    self.ctx._operands = ['/tmp', 'a']

    self.assertRaises(gvn.errors.MixedPaths, self.ctx.DisplayPath, '')
    self.assertRaises(gvn.errors.MixedPaths, lambda: self.ctx.wc)
    self.assertRaises(gvn.errors.MixedPaths, lambda: self.ctx.project)

  def testWcOperandsAbsolute(self):
    # Move up out of the wc.
    os.chdir('..')
    # Now specify absolute path to the wc.
    self.ctx._operands = [self.wcdir]

    # Note that this only tests something on Windows; on unix it's /
    # no matter what.
    self.assertEquals(self.ctx.operands, [self.wcdir_internal_style])

    self.assertEquals(self.ctx.DisplayPath('greek-tree/A'),
                      os.path.join(os.path.basename(self.wcdir),
                                   'greek-tree', 'A'))
    self.assertEquals(self.ctx.wc.path, self.wcdir_internal_style)
    self.assertEquals(self.ctx.project.path, 'public/projects')

  def testWcOperandsRelative(self):
    self.ctx._operands = [os.path.join('greek-tree', 'A')]

    # Note that this only tests something on Windows; on unix it's /
    # no matter what.
    self.assertEquals(self.ctx.operands, ['greek-tree/A'])

    self.assertEquals(self.ctx.DisplayPath('greek-tree/A'),
                                           os.path.join('greek-tree', 'A'))
    # XXX i think this is supposed to be ''; see issue 12
    self.assertEquals(self.ctx.DisplayPath(''), '.')
    self.assertEquals(self.ctx.wc.path, self.wcdir_internal_style)
    self.assertEquals(self.ctx.project.path, 'public/projects')

  def SetAuth(self):
    testing.testcommon.GetAuthBaton(self.config, None,
                                    self.ctx.project.repository)

  def testProject(self):
    # Test --project NAME with root.
    self.ctx.options.project = 'root'
    self.SetAuth()
    self.assertEquals(self.ctx.project.path, '')
    self.assertEquals(self.ctx.project.repository.URL(), self.repo_url)

    # Test --project NAME with subdir.
    self.Ctx().options.project = 'pub'
    self.SetAuth()
    self.assertEquals(self.ctx.project.path, 'public/projects')
    self.assertEquals(self.ctx.project.repository.URL(), self.repo_url)

    # Test --project URL with root.
    self.Ctx().options.project = self.repo_url
    self.SetAuth()
    self.assertEquals(self.ctx.project.path, '')
    self.assertEquals(self.ctx.project.repository.URL(), self.repo_url)

    # Test --project URL with subdir.
    self.Ctx().options.project = self.repo_url + '/public/projects'
    self.SetAuth()
    self.assertEquals(self.ctx.project.path, 'public/projects')
    self.assertEquals(self.ctx.project.repository.URL(), self.repo_url)

    # Test project from WC.
    self.Ctx()
    self.SetAuth()
    self.assertEquals(self.ctx.project.path, 'public/projects')
    self.assertEquals(self.ctx.project.repository.URL(), self.repo_url)

    # Create default project file with yet a third URL (make sure this
    # URL is not the same as either URL in setUp).
    fp = open(os.path.join(self.projects_dir, 'default'), 'w')
    fp.write('[project]\n'
             'url = %s\n' % (self.repo_url + '/public/projects/greek-tree',))
    fp.close()

    # Test project default by leaving the WC.
    os.chdir('..')
    self.Ctx()
    self.SetAuth()
    self.assertEquals(self.ctx.project.path, 'public/projects/greek-tree')
    self.assertEquals(self.ctx.project.repository.URL(), self.repo_url)
