# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import shutil
import unittest

import gvn.config
import gvn.errors
from gvn import platform
from testing import testcommon


class TestGvnConfig(unittest.TestCase):

  def testBasic(self):
    config = gvn.config.Config()
    self.assertEquals(config.project_dirs, [])

    # Crazy assertion that every property have a corresponding
    # default key in config._data in order to help assure that
    # no new properties are added without defining a corresponding
    # Config._data entry.
    for attr in dir(gvn.config.Config):
      if type(getattr(gvn.config.Config, attr)) is property:
        self.assertTrue(attr in config._data, "no _data key '%s'" % (attr,))

    config.smtp_server = 'foo'
    self.assertEquals(config.smtp_server, 'foo')

    os.mkdir('cftest')

    # Test empty file.
    config = gvn.config.Config()
    self.assertEqual(None, config.smtp_server)
    self.assertEqual(None, config.diff_command)
    open('cftest/config', 'w').write('')
    gvn.config._Load(config, 'cftest')
    self.assertEqual(None, config.smtp_server)
    self.assertEqual(None, config.diff_command)
    self.assertEquals(config.project_dirs, [os.path.join('cftest', 'projects')])

    # Test content-free file.
    config = gvn.config.Config()
    self.assertEqual(None, config.smtp_server)
    self.assertEqual(None, config.diff_command)
    open('cftest/config', 'w').write("""[external]
#diff_command = tkdiff
""")
    gvn.config._Load(config, 'cftest')
    self.assertEqual(None, config.smtp_server)
    self.assertEqual(None, config.diff_command)
    self.assertEquals(config.project_dirs, [os.path.join('cftest', 'projects')])

    # Test complete file.
    open('cftest/config', 'w').write("""[external]
smtp_server: server
diff_command = tkdiff
""")
    config = gvn.config.Config()
    gvn.config._Load(config, 'cftest')
    self.assertEquals(config.smtp_server, 'server')
    self.assertEquals(config.diff_command, 'tkdiff')
    self.assertEquals(config.project_dirs, [os.path.join('cftest', 'projects')])
    self.assertRaises(gvn.errors.NoProject, config.ProjectFile, 'foo')
    self.assertRaises(gvn.errors.NoProject, config.ProjectByName, 'foo')
    self.assertRaises(gvn.errors.NoProject, config.ProjectByURL, 'foo')

    pdir1 = os.path.join(os.getcwd(), 'pdir1')
    pdir2 = os.path.join(os.getcwd(), 'pdir2')
    os.mkdir(pdir1)
    os.mkdir(pdir2)
    config.project_dirs.append(pdir1 + '/projects')
    config.project_dirs.append(pdir2 + '/projects')

    os.mkdir('pdir1/projects')
    open('pdir1/projects/p1', 'w').write("""[project]
url=url1
username=bob
""")
    project = config.ProjectByName('p1')
    self.assertEquals(project.URL, 'url1')
    self.assertEquals(project.username, 'bob')
    project = config.ProjectByURL('url1')
    self.assertEquals(project.URL, 'url1')
    self.assertEquals(project.username, 'bob')
    project = config.ProjectByURL('url1/foo')
    self.assertEquals(project.URL, 'url1')
    self.assertEquals(project.username, 'bob')
    self.assertRaises(gvn.errors.NoProject, config.ProjectByURL, 'foo')
    self.assertRaises(gvn.errors.NoProject, config.ProjectDefault)

    gvn.config._GetDefaultUser = testcommon.UnittestUser
    gvn.config._GetDefaultSvnUserConfigDir = lambda: '.subversion'
    gvn.config._GetDefaultSvnSystemConfigDir = lambda: os.path.join('etc',
                                                                  'subversion')
    gvn.config._GetDefaultGvnUserConf = lambda: 'gvn'
    gvn.config._GetDefaultGvnSystemConf = lambda: os.path.join('etc', 'gvn')
    gvn.config._GetDomainName = lambda: 'example.com'

    config = gvn.config.Get()
    self.assertEquals(config.svn_config_dir, '.subversion')
    self.assertEquals(config.diff_command, 'internal')
    self.assertEquals(config.smtp_server, 'smtp')
    self.assertEquals(config.project_dirs, [os.path.join('gvn', 'projects'),
                                       os.path.join('etc', 'gvn', 'projects')])
    self.assertRaises(gvn.errors.NoProject, config.ProjectDefault)

    os.makedirs('etc/gvn/projects')
    os.makedirs('gvn/projects')

    config = gvn.config.Get()
    open('gvn/projects/p1', 'w').write("""[project]
url=url1
""")
    project = config.ProjectByName('p1')
    self.assertEquals(project.URL, 'url1')
    self.assertEquals(project.username, testcommon.UnittestUser())
    project = config.ProjectByURL('url1')
    self.assertEquals(project.URL, 'url1')
    self.assertEquals(project.username, testcommon.UnittestUser())
    project = config.ProjectByURL('url1/foo')
    self.assertEquals(project.URL, 'url1')
    self.assertEquals(project.username, testcommon.UnittestUser())
    self.assertRaises(gvn.errors.NoProject, config.ProjectByURL, 'foo')
    self.assertRaises(gvn.errors.NoProject, config.ProjectDefault)
    # Windows has no symlink, so we copy the project file instead.
    shutil.copyfile('gvn/projects/p1', 'gvn/projects/default')
    project = config.ProjectDefault()
    self.assertEquals(project.URL, 'url1')
    self.assertEquals(project.username, testcommon.UnittestUser())

    self.assertEquals(project.email_address, 'unittest@example.com')
    os.environ['EMAIL'] = 'env-test'
    self.assertEquals(project.email_address, 'env-test')
    os.environ['EMAIL'] = '  space -env-test	'
    self.assertEquals(project.email_address, 'space -env-test')
    os.environ['EMAIL'] = ''
    self.assertEquals(project.email_address, 'unittest@example.com')
    del os.environ['EMAIL']
    self.assertEquals(project.email_address, 'unittest@example.com')
    project.email_address = 'foo'
    self.assertEquals(project.email_address, 'foo')

    self.assertEquals(config.editor_command, platform.DEFAULT_EDITOR_COMMAND)
    os.environ['EDITOR'] = 'foo --bar'
    self.assertEquals(config.editor_command, platform.DEFAULT_EDITOR_COMMAND)
    config = gvn.config.Get()
    self.assertEquals(config.editor_command, 'foo --bar')

  def testProjectByURL(self):
    # Create config and projects dir.
    pdir = os.path.join(os.getcwd(), 'testProjectByURL')
    os.mkdir(pdir)
    config = gvn.config.Config()
    config.project_dirs.append(pdir)

    # Create project file with URL of repository root.
    fp = open(os.path.join(pdir, 'a'), 'w')
    fp.write('[project]\n'
             'url = http://svn/r\n')
    fp.close()

    # Create project file with URL of public/projects.
    fp = open(os.path.join(pdir, 'b'), 'w')
    fp.write('[project]\n'
             'url = http://svn/r/public/projects')
    fp.close()

    # Ensure ProjectByURL finds the *longest* match.  It used to
    # return the first match of any kind, so (above) we store the
    # wrong match as 'a' and the right one as 'b' and pass
    # testing=True so it will sort by name and we know it will hit the
    # wrong one first.
    for (input, expected_output) in [
      ('http://svn/r/public',                  'http://svn/r'),
      ('http://svn/r/public/projectsfoo',      'http://svn/r'),
      ('http://svn/r/public/projects',         'http://svn/r/public/projects'),
      ('http://svn/r/public/projects/foo',     'http://svn/r/public/projects'),
      ]:
      project = config.ProjectByURL(input, testing=True)
      self.assertEquals(project.URL, expected_output,
                        msg="'%s' != '%s' (input='%s')"
                            % (project.URL, expected_output, input))

  def testInvalidProjectNames(self):
    """Separate test to check that invalid project filenames are
    appropriately handled.

    """

    config = gvn.config.Get()
    config.project_dirs = ['testdata/config/projects']

    # Be sure there are some good and some bad project file names:
    filenames = {'good': 0, 'bad': 0}
    for filename in os.listdir(config.project_dirs[0]):
      if not gvn.config.PROJECT_FILENAME_RE.match(filename):
        filenames['bad'] += 1
      else:
        filenames['good'] += 1
    self.assertEquals({'good': 2, 'bad': 3}, filenames)

    # Calling ProjectByURL exercises the code that walks the directory
    # and should encounter the above bogus filenames.
    project = config.ProjectByURL('file:///foo/repo')
    self.assertEquals('foo_user', project.username)
    self.assertRaises(gvn.errors.NoProject, config.ProjectByURL,
                      'file:///quux/repo')

    self.assertRaises(gvn.errors.InvalidProjectName,
                      config.ProjectByName, '')
    self.assertRaises(gvn.errors.InvalidProjectName,
                      config.ProjectByName, 'bogus~')
    self.assertRaises(gvn.errors.InvalidProjectName,
                      config.ProjectByName, '.#bogus#')
    self.assertRaises(gvn.errors.InvalidProjectName,
                      config.ProjectByName, '.bogus.swp')
