# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import unittest

import gvn.userdb

import testing.tmpdir


class TestUserDB(unittest.TestCase):
  def setUp(self):
    self.dir = testing.tmpdir.RandomTmpSubdir()
    self.db = gvn.userdb.UserDB(self.dir)

  def test__init__(self):
    self.assertEqual(gvn.userdb.UserDB(self.dir)._dirpath, self.db._dirpath)
    self.assertEqual(gvn.userdb.UserDB(self.dir, create_db=True)._dirpath,
                     self.db._dirpath)
    os.rmdir(self.dir)
    self.assertEqual(gvn.userdb.UserDB(self.dir, create_db=True)._dirpath,
                     self.db._dirpath)

  def testReadWriteUserPasswordAndDelete(self):
    """ReadUserPassword, WriteUserPassword, DeleteUser, RandomizeUserPassword"""
    self.assertRaises(gvn.userdb.NoUserError,
                      self.db.ReadUserPassword, 'polly')
    self.db.WriteUserPassword('polly', 'hey')
    (user, cleartext, crypted) = self.db.ReadUserPassword('polly')
    self.assertEqual((user, cleartext), ('polly', 'hey'))
    self.assertTrue(len(crypted) >= 0)
    self.db.DeleteUser('polly')
    self.assertRaises(gvn.userdb.NoUserError,
                      self.db.ReadUserPassword, 'polly')
    self.db.DeleteUser('polly')
    self.db.RandomizeUserPassword('polly')
    (user, cleartext, crypted) = self.db.ReadUserPassword('polly')
    self.assertTrue(len(cleartext) >= 0)
    self.assertTrue(len(crypted) >= 0)

  def testDoesUserExist(self):
    self.db.WriteUserGroups('polly', [ 'sucker' ])
    self.assertTrue(self.db.DoesUserExist('polly'))
    self.assertFalse(self.db.DoesUserExist('polly', require_password=True))

    self.db.WriteUserPassword('polly', 'hey')
    self.assertTrue(self.db.DoesUserExist('polly', require_password=True))

  def testReadWriteUserGroups(self):
    groups = ['artists', 'waiters']
    self.db.WriteUserGroups('polly', groups)
    self.assertEqual(self.db.ReadUserGroups('polly'), groups)

  # This UT tests both GetAllUsers and CreateSvnOnlyUser
  def testGetAllUsers(self):
    groups = ['artists', 'waiters']
    self.db.WriteUserGroups('polly', groups)
    self.db.WriteUserPassword('sybil', 'flyingtart')
    self.db.CreateSvnOnlyUser('basil')
    self.assertEqual(sorted(self.db.GetAllUsers()), ['basil', 'sybil'])
    self.assertEqual(sorted(self.db.GetAllUsers(require_password=True)),
                     ['basil', 'sybil'])
    self.assertEqual(sorted(self.db.GetAllUsers(require_password=False)),
                     ['basil', 'polly', 'sybil'])
    self.assertEqual(sorted(self.db.GetAllUsers(require_password=False,
                     return_not_autosynced=False)),
                     ['polly', 'sybil'])

  def testDeleteUsers(self):
    self.db.WriteUserPassword('sybil', 'flyingtart')
    self.assertEqual(sorted(self.db.GetAllUsers()), ['sybil'])
    self.db.DeleteUser('sybil')
    self.assertEqual(sorted(self.db.GetAllUsers()), [])

  def testLockUser(self):
    self.db.WriteUserPassword('polly', 'flyingtart')

    self.db.LockUser('polly')
    (username, plain, crypted) = self.db.ReadUserPassword('polly')
    self.assertEqual(username, 'polly')
    self.assertTrue(plain.startswith('!!'))
    self.assertTrue(crypted.startswith('!!'))

    self.db.UnLockUser('polly')
    (username, plain, crypted) = self.db.ReadUserPassword('polly')
    self.assertEqual(username, 'polly')
    self.assertFalse(plain.startswith('!!'))
    self.assertFalse(crypted.startswith('!!'))
