# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import posixpath
import unittest

import svn.core
import svn.wc

from svn.core import SWIG_SVN_INVALID_REVNUM as SVN_INVALID_REVNUM
from svn.core import svn_node_dir, svn_node_file
from svn.core import svn_path_internal_style

import gvn.commit
import gvn.errors
import gvn.platform
import gvn.svncmd
import gvn.wc

from testing import testcommon


class TestCase(unittest.TestCase):
  # TODO(epg): We probably want some common class in testcommon...
  def setUp(self):
    self.config = testcommon.GetConfig()
    self.repo = testcommon.NewTmpRepository()
    self.repo_url = testcommon.TmpRepositoryURL()
    self.wcdir = testcommon.NewWorkingCopy()
    os.chdir(self.wcdir)
    self.assert_(os.path.isdir('./.svn'))

class TestWorkingCopy(TestCase):
  def testBasic(self):
    pool = svn.core.Pool()
    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=self.config, pool=pool)
    self.assertEquals(wc.Open(write_lock=False), None)
    self.assertRaises(gvn.errors.WCReOpen, wc.Open, write_lock=True)
    self.assertEquals(wc.path, self.wcdir)

    self.assertEquals(wc.project.path, 'public/projects')

    self.assertEquals(wc.project.repository.URL(), self.repo_url)
    self.assertEquals(wc.AbsolutePath('greek-tree/A/B'),
                      self.wcdir + '/greek-tree/A/B')
    self.assertEquals(wc.RelativePath(self.wcdir + '/greek-tree/A/B'),
                      'greek-tree/A/B')
    self.assertRaises(gvn.errors.PathNotChild,
                      wc.RelativePath, 'breakit')
    self.assertRaises(gvn.errors.PathNotChild,
                      wc.RelativePath, 'breakit' + self.wcdir)

  def testLoadSave(self):
    pool = svn.core.Pool()
    dot_gvn = os.path.join(self.wcdir, gvn.wc.STATE_FILE)

    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=self.config, pool=pool)
    self.assertEquals(wc.change_state, {})
    self.assertFalse(os.path.exists(dot_gvn))

    wc.Save()
    self.assertTrue(os.path.exists(dot_gvn))

    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=self.config, pool=pool)
    self.assertEquals(wc.change_state, {})

    class mock_entry:
      kind = copyfrom_url = copyfrom_rev = has_prop_mods = None
    class mock_status:
      entry = mock_entry()
      text_status = prop_status = svn.wc.status_deleted
    wc.change_state['greek-tree'] = gvn.wc.ChangeState('changebranch', None,
                                                       None, mock_status())
    wc.Save()
    self.assertTrue(os.path.exists(dot_gvn))

    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=self.config, pool=pool)
    self.assertEquals(wc.change_state.keys(), ['greek-tree'])
    self.assertEquals(wc.change_state['greek-tree'].change_name, 'changebranch')

  def testBasicOpen(self):
    pool = svn.core.Pool()
    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=self.config, pool=pool)

    self.assertRaises(gvn.errors.WCClosed, wc.Entry, '', pool=pool)
    self.assertRaises(gvn.errors.WCClosed, wc.Close)

    def url(p):
      return '/'.join([self.repo_url, 'public/projects', p]).rstrip('/')

    # default (open .path recursively)
    wc.Open()
    self.assertEquals(wc.subpath, '')
    self.assertEquals(wc.Entry(pool=pool).url, url(''))
    self.assertEquals(wc.Entry('', pool=pool).url, url(''))
    self.assertEquals(wc.Entry('greek-tree', pool=pool).url, url('greek-tree'))
    self.assertEquals(wc.Entry('greek-tree/iota', pool=pool).url,
                      url('greek-tree/iota'))
    wc.Close()

    # open .path non-recursively
    wc.Open(recursive=False)
    self.assertEquals(wc.subpath, '')
    self.assertEquals(wc.Entry(pool=pool).url, url(''))
    self.assertEquals(wc.Entry('', pool=pool).url, url(''))
    self.assertEquals(wc.Entry('greek-tree', pool=pool).url, None)
    self.assertEquals(wc.Entry('greek-tree/iota', pool=pool), None)
    wc.Close()

    # open 'greek-tree' recursively
    wc.Open(['greek-tree'])
    self.assertEquals(wc.subpath, 'greek-tree')
    self.assertEquals(wc.Entry(pool=pool).url, url('greek-tree'))
    self.assertEquals(wc.Entry('', pool=pool).url, url(''))
    self.assertEquals(wc.Entry('greek-tree', pool=pool).url, url('greek-tree'))
    self.assertEquals(wc.Entry('greek-tree/iota', pool=pool).url,
                      url('greek-tree/iota'))
    wc.Close()

    # open 'greek-tree' non-recursively
    wc.Open(['greek-tree'], recursive=False)
    self.assertEquals(wc.subpath, 'greek-tree')
    self.assertEquals(wc.Entry(pool=pool).url, url('greek-tree'))
    self.assertEquals(wc.Entry('', pool=pool).url, url(''))
    self.assertEquals(wc.Entry('greek-tree', pool=pool).url, url('greek-tree'))
    self.assertEquals(wc.Entry('greek-tree/iota', pool=pool).url,
                      url('greek-tree/iota'))
    self.assertEquals(wc.Entry('greek-tree/A', pool=pool).url, None)
    self.assertEquals(wc.Entry('greek-tree/A/B', pool=pool), None)
    wc.Close()

    # open 'greek-tree/A' recursively
    wc.Open(['greek-tree/A'])
    self.assertEquals(wc.subpath, 'greek-tree/A')
    self.assertEquals(wc.Entry(pool=pool).url, url('greek-tree/A'))
    self.assertEquals(wc.Entry('', pool=pool), None)
    wc.Close()

    # open 'greek-tree/A' non-recursively
    wc.Open(['greek-tree/A'], recursive=False)
    self.assertEquals(wc.subpath, 'greek-tree/A')
    self.assertEquals(wc.Entry(pool=pool).url, url('greek-tree/A'))
    self.assertEquals(wc.Entry('', pool=pool), None)
    wc.Close()

  def testOpenAdded(self):
    # test Open of Added directories
    pool = svn.core.Pool()
    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=self.config, pool=pool)

    def url(p):
      return '/'.join([self.repo_url, 'public/projects', p]).rstrip('/')

    wc.Open(write_lock=True)
    os.mkdir('new')
    testcommon.WCAdd(wc, 'new', pool)
    os.mkdir('greek-tree/A/B/new')
    testcommon.WCAdd(wc, 'greek-tree/A/B/new', pool)
    wc.Close()

    wc.Open(['new'])
    self.assertNotEquals(wc.Entry(pool=pool).revision, SVN_INVALID_REVNUM)
    self.assertEquals(wc.subpath, '')
    self.assertEquals(wc.Entry(pool=pool).url, url(''))
    self.assertEquals(wc.Entry('', pool=pool).url, url(''))
    wc.Close()

    wc.Open(['greek-tree/A/B/new'])
    self.assertNotEquals(wc.Entry(pool=pool).revision, SVN_INVALID_REVNUM)
    self.assertEquals(wc.subpath, 'greek-tree/A/B')
    self.assertEquals(wc.Entry(pool=pool).url, url('greek-tree/A/B'))
    self.assertEquals(wc.Entry('', pool=pool), None)
    wc.Close()

  def testProject(self):
    pdir = os.path.join(os.getcwd(), 'pdir', 'projects')
    os.mkdir('pdir'); os.mkdir(pdir)
    open('pdir/projects/p1', 'w').write("""[project]
url=%s/public
""" % (self.repo_url,))
    open('pdir/projects/default', 'w').write("""[project]
url=http://foo/bar
""")
    config = testcommon.GetConfig()
    config.project_dirs.append(pdir)
    pool = svn.core.Pool()
    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=config, pool=pool)
    self.assertEquals(wc.project.path, 'public')
    self.assertEquals(wc.project.repository.URL(), self.repo_url)

  def testStatus(self):
    pool = svn.core.Pool()
    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=self.config, pool=pool)
    wc.Open(['greek-tree'])

    ((path, st),) = wc.Status(['greek-tree/iota'])
    self.assertEquals(st.entry.url, wc._url + '/greek-tree/iota')
    self.assertEquals(st.text_status, svn.wc.status_normal)

    ((path, st),) = wc.Status(['greek-tree'])
    self.assertEquals(st.entry.url, wc._url + '/greek-tree')
    self.assertEquals(st.text_status, svn.wc.status_normal)

    (exitcode, out, err) = gvn.svncmd.CaptureSvn(['ls', '-R', 'greek-tree'])
    self.assertEquals(exitcode, 0)
    # status includes greek-tree, ls does not, so + 1.
    n = len([x for x in out.splitlines()]) + 1
    status = wc.Status(['greek-tree'], recursive=True)
    self.assertEquals(len(status), n)

    ((path, st),) = wc.Status(['greek-tree/A/B/lambda'])
    self.assertEquals(st.entry.url, wc._url + '/greek-tree/A/B/lambda')
    self.assertEquals(st.text_status, svn.wc.status_normal)

    (exitcode, out, err) = gvn.svncmd.CaptureSvn(['ls', '-R'])
    self.assertEquals(exitcode, 0)
    # status includes ., ls does not, so + 1.
    n = len([x for x in out.splitlines()]) + 1
    status = wc.Status(recursive=True)
    self.assertEquals(len(status), n)

    open('greek-tree/iota', 'a').write('new line\n')
    ((path, st),) = wc.Status(['greek-tree/iota'])
    self.assertEquals(st.text_status, svn.wc.status_modified)

  def testStatusShowChildren(self):
    pool = svn.core.Pool()
    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=self.config, pool=pool)
    wc.Open(write_lock=True)

    # Create a new tree; show_children=False should show the whole tree.
    os.mkdir('foobar')
    svn.wc.add2(wc.AbsolutePath('foobar'),
                svn.wc.adm_probe_retrieve(wc._adm_access,
                                    os.path.dirname(wc.AbsolutePath('foobar')),
                                          pool),
                None,                   # copyfrom_url
                -1,                     # copyfrom_rev
                None,                   # cancel_func
                None,                   # notify_func
                pool)
    open('foobar/file', 'w')
    testcommon.WCAdd(wc, 'foobar/file', pool)

    # Copy greek-tree to foobar2; show_children=False should show only foobar2.
    testcommon.WCCopy(wc, 'greek-tree', 'foobar2', pool)

    # Here's a random other edit to throw into the mix.
    testcommon.WCDelete(wc, 'greek-tree/A/B', pool)

    # Now check Status.
    wc.Close()
    wc.Open(write_lock=False)
    self.assertEquals(sorted(wc.RelativePath(target) for (target, status) in
                             wc.Status(recursive=True, show_children=False,
                                       get_all=False)),
                      ['foobar', 'foobar/file', 'foobar2',
                       'greek-tree/A/B'])

    # Test show_children=False for unversioned files.
    # XXX(epg): What was the point of this one?
    open('unversioned', 'w')
    self.assertEquals(len(wc.Status(['unversioned'], show_children=False)), 1)

    # Now let's get more complicated, and cover a case that used to
    # break gvn: copy a tree and then add and/or remove paths under
    # the copied-to tree.  We used to hide the added and removed
    # paths, but we need to show them.  See testDiffCpThenRm in
    # test_changebranch for more; see TODO for submit (still broken).
    wc.Close()
    wc.Open(write_lock=True)
    testcommon.WCCopy(wc, 'greek-tree/A/D', 'acopy', pool)
    testcommon.WCCopy(wc, 'acopy/gamma', 'acopy/gamma-hey', pool)
    testcommon.WCDelete(wc, 'acopy/gamma', pool)
    wc.Close()
    wc.Open(write_lock=False)
    self.assertEquals(sorted(wc.RelativePath(target) for (target, status) in
                             wc.Status(['acopy'], recursive=True,
                                       show_children=False, get_all=False)),
                      ['acopy', 'acopy/gamma', 'acopy/gamma-hey'])

  def testPaths(self):
    pool = svn.core.Pool()
    wc = gvn.wc.WorkingCopy(self.wcdir, cancel_func=None, notify_func=None,
                     config=self.config, pool=pool)

    self.assertEquals(wc.AbsolutePath(''), self.wcdir)
    self.assertEquals(wc.AbsolutePath('foo'), self.wcdir + '/foo')
    # bogus input => bogus output
    self.assertEquals(wc.AbsolutePath('/foo'), self.wcdir + '//foo')

    self.assertEquals(wc.RelativePath(self.wcdir), '')
    self.assertEquals(wc.RelativePath(self.wcdir + '/foo'), 'foo')
    # bogus input => exception
    self.assertRaises(gvn.errors.PathNotChild, wc.RelativePath, '/etc')

    self.assertEquals(wc.RepoPath(), 'public/projects')
    self.assertEquals(wc.RepoPath(''), 'public/projects')
    self.assertEquals(wc.RepoPath('foo'), 'public/projects/foo')
    # bogus input => bogus output
    self.assertEquals(wc.RepoPath('/foo'), 'public/projects//foo')

    self.assertEquals(wc.URL(), self.repo_url + '/public/projects')
    self.assertEquals(wc.URL(''), self.repo_url + '/public/projects')
    self.assertEquals(wc.URL('foo'), self.repo_url + '/public/projects/foo')
    # bogus input => bogus output
    self.assertEquals(wc.URL('/foo'), self.repo_url + '/public/projects//foo')

    self.assertEquals(wc.LocalPath('public/projects'), '')
    self.assertEquals(wc.LocalPath('public/projects/foo'), 'foo')
    self.assertEquals(wc.LocalPath(self.repo_url + '/public/projects'), '')
    self.assertEquals(wc.LocalPath(self.repo_url + '/public/projects/foo'),
                      'foo')
    # bogus input => exception
    self.assertRaises(gvn.errors.PathNotChild,
                      wc.LocalPath, '/public/projects')
    self.assertRaises(gvn.errors.PathNotChild,
                      wc.LocalPath, self.repo_url + 'public/projects')

  def testFindWorkingCopy(self):
    pool = svn.core.Pool()

    # very basic, just pass it the top
    (wc, subpath) = gvn.wc.FindWorkingCopy(self.wcdir,
                                    cancel_func=None, notify_func=None,
                                    config=self.config, pool=pool)
    self.assertEquals(subpath, '')
    self.assertEquals(wc.path, self.wcdir)

    # now pass it a subdir
    (wc, subpath) = gvn.wc.FindWorkingCopy(self.wcdir + '/greek-tree',
                                    cancel_func=None, notify_func=None,
                                    config=self.config, pool=pool)
    self.assertEquals(subpath, 'greek-tree')
    self.assertEquals(wc.path, self.wcdir)

    # now an unversioned directory
    os.mkdir('new-directory')
    (wc,
     subpath) = gvn.wc.FindWorkingCopy(self.wcdir + '/new-directory',
                                cancel_func=None, notify_func=None,
                                config=self.config, pool=pool)
    self.assertEquals(subpath, 'new-directory')
    self.assertEquals(wc.path, self.wcdir)

    # now an unversioned directory, two levels down
    os.mkdir('new-directory/a')
    (wc,
     subpath) = gvn.wc.FindWorkingCopy(self.wcdir + '/new-directory/a',
                                cancel_func=None, notify_func=None,
                                config=self.config, pool=pool)
    wc.Open()
    self.assertEquals(subpath, 'new-directory/a')
    self.assertEquals(wc.path, self.wcdir)

    # an added directory
    wc.Close(); wc.Open(write_lock=True)
    testcommon.WCAdd(wc, 'new-directory', pool)
    wc.Close(); wc.Open(write_lock=False)
    (wc,
     subpath) = gvn.wc.FindWorkingCopy(self.wcdir + '/new-directory',
                                cancel_func=None, notify_func=None,
                                config=self.config, pool=pool)
    self.assertEquals(subpath, 'new-directory')
    self.assertEquals(wc.path, self.wcdir)

    # a directory that's not a working copy at all: our temporary wc's parent
    self.assertRaises(gvn.errors.NotWC, gvn.wc.FindWorkingCopy,
                      posixpath.dirname(self.wcdir), cancel_func=None,
                      notify_func=None, config=self.config, pool=pool)

class TestEdit(TestCase):
  def testBasic(self):
    # TODO(epg): Test prop change, _Added, and _Replaced.
    pool = svn.core.Pool()
    (wc, subpath) = gvn.wc.FindWorkingCopy(svn_path_internal_style(os.getcwd(),
                                                                   pool),
                                           cancel_func=None, notify_func=None,
                                    config=self.config, pool=pool)
    testcommon.GetAuthBaton(self.config, pool, wc.project.repository)
    wc.Open(['greek-tree'], write_lock=True)

    # Modify and commit iota.
    iota = 'greek-tree/iota'
    testcommon.sleepf(lambda: open(iota, 'a').write('line\n'))
    deltaman = gvn.wc.PostfixTextDeltaManager(wc)
    commit_info = gvn.commit.Drive(wc.project.repository, {'svn:log': ''},
                                   SVN_INVALID_REVNUM,
                                   [wc.RepoPath(iota)],
                                   (lambda *a: gvn.wc.Edit(wc, iota,
                                                     wc.Status([iota])[0][1],
                                                           deltaman.Add)),
                                   deltaman.Transmit, pool)
    wc.ProcessCommittedQueue(commit_info, pool)
    wc.Close()
    (revision,) = wc.project.repository.GetRevisions(start=commit_info.revision)
    self.assertEquals([x.path for x in revision.paths],
                      ['/public/projects/greek-tree/iota'])
    self.assertEquals(revision.paths[0].action, 'M')
    wc.Open(write_lock=False)
    ((path, st),) = wc.Status([iota])
    self.assertEquals(st.text_status, svn.wc.status_normal)
    self.assertEquals(st.entry.revision, commit_info.revision)
    wc.Close()

    # Modify iota again, delete lambda, and commit them (ensuring we
    # can commit to the same file twice).
    wc.Open(write_lock=True)
    ablambda = 'greek-tree/A/B/lambda'
    svn.wc.delete2(wc.AbsolutePath(ablambda),
                   svn.wc.adm_probe_retrieve(wc._adm_access,
                                             wc.AbsolutePath(ablambda), pool),
                   None,                # cancel_func
                   None,                # notify_func
                   pool)
    testcommon.sleepf(lambda: open(iota, 'a').write('line\n'))
    deltaman = gvn.wc.PostfixTextDeltaManager(wc)
    def action(path, pool):
      wc_path = wc.LocalPath(path)
      return gvn.wc.Edit(wc, wc_path, wc.Status([wc_path])[0][1], deltaman.Add)
    commit_info = gvn.commit.Drive(wc.project.repository, {'svn:log': ''},
                                   SVN_INVALID_REVNUM,
                                   [wc.RepoPath(iota),
                                    wc.RepoPath(ablambda)],
                                   action, deltaman.Transmit, pool)
    wc.ProcessCommittedQueue(commit_info, pool)
    wc.Close()
    (revision,) = wc.project.repository.GetRevisions(start=commit_info.revision)
    paths = sorted(revision.paths, key=lambda x: x.path)
    self.assertEquals([x.path for x in paths],
                      ['/public/projects/greek-tree/A/B/lambda',
                       '/public/projects/greek-tree/iota'])
    self.assertEquals(paths[0].action, 'D')
    self.assertEquals(paths[1].action, 'M')
    wc.Open(write_lock=False)
    ((path, st),) = wc.Status([iota])
    self.assertEquals(st.text_status, svn.wc.status_normal)
    self.assertEquals(st.entry.revision, commit_info.revision)

    # When wcprop callbacks are not getting called for whatever reason
    # (e.g. until r24937 (see CorpSvnDiffs) they were broken), our
    # submits don't update wcprops.  This breaks subsequent commits
    # (mostly; see TODO(epg) below).

    # Edit iota again, to commit and find the wcprop problem.
    testcommon.sleepf(lambda: open(iota, 'a').write('line\n'))

    # TODO(epg): It would be nice if i could find a reliable way to
    # make our own commit code fail in this case.  I don't know why we
    # can get away without wcprops but svn can't...  In the mean time,
    # run svn to find the problem.
    (code, stdout, stderr) =  gvn.svncmd.CaptureSvn(['commit', '-m', 'msg'])
    if code != 0 and 'probably out-of-date' in stderr:
      self.fail('gvn.repository.ra_callbacks wcprop callbacks broken')

  def testRecursiveAdd(self):
    pool = svn.core.Pool()
    (wc, subpath) = gvn.wc.FindWorkingCopy(svn_path_internal_style(os.getcwd(),
                                                                   pool),
                                    cancel_func=None, notify_func=None,
                                    config=self.config, pool=pool)
    testcommon.GetAuthBaton(self.config, pool, wc.project.repository)
    wc.Open(['greek-tree'], write_lock=True)
    deltaman = gvn.wc.PostfixTextDeltaManager(wc)
    def action(path, pool):
      wc_path = wc.LocalPath(path)
      return gvn.wc.Edit(wc, wc_path, wc.Status([wc_path])[0][1], deltaman.Add)

    # Make a copy.
    testcommon.sleepf(testcommon.WCCopy, wc, 'greek-tree', 'foo', pool)
    deltaman = gvn.wc.PostfixTextDeltaManager(wc)
    commit_info = gvn.commit.Drive(wc.project.repository, {'svn:log': ''},
                                   SVN_INVALID_REVNUM,
                                   [wc.RepoPath('foo')], action,
                                   deltaman.Transmit, pool)
    wc.ProcessCommittedQueue(commit_info, pool)
    wc.Close()
    wc.Open(write_lock=False)
    self.assertEquals(wc.Entry('foo', pool).revision, commit_info.revision)
    wc.Close()

    # Make a copy into that, and add a new tree under it.
    paths = ['foo/foo', 'foo/foo/RecursiveAdd', 'foo/foo/RecursiveAdd/dir1',
             'foo/foo/RecursiveAdd/file1', 'foo/foo/RecursiveAdd/dir1/file2']
    wc.Open(write_lock=True)
    testcommon.WCCopy(wc, 'greek-tree', paths[0], pool)
    os.mkdir(paths[1])
    os.mkdir(paths[2])
    open(paths[3], 'w').write('hi\n')
    testcommon.sleepf(open(paths[4], 'w').write, 'bye\n')
    for i in paths[1:]:
      testcommon.WCAdd(wc, i, pool)
    deltaman = gvn.wc.PostfixTextDeltaManager(wc)
    commit_info = gvn.commit.Drive(wc.project.repository, {'svn:log': ''},
                                   SVN_INVALID_REVNUM,
                                   [wc.RepoPath(x) for x in paths],
                                   action, deltaman.Transmit, pool)
    wc.ProcessCommittedQueue(commit_info, pool)
    wc.Close()
    wc.Open(write_lock=False)
    for i in paths:
      self.assertEquals(wc.Entry(i, pool).revision, commit_info.revision)
    wc.Close()

    # Modify RecursiveAdd/dir1/file2 again.
    wc.Open(write_lock=True)
    testcommon.sleepf(open(paths[4], 'a').write, 'new line\n')
    deltaman = gvn.wc.PostfixTextDeltaManager(wc)
    commit_info = gvn.commit.Drive(wc.project.repository, {'svn:log': ''},
                                   SVN_INVALID_REVNUM,
                                   [wc.RepoPath(paths[4])], action,
                                   deltaman.Transmit, pool)
    wc.ProcessCommittedQueue(commit_info, pool)
    wc.Close()
    wc.Open(write_lock=False)
    self.assertEquals(wc.Entry(paths[4], pool).revision, commit_info.revision)

  def testRecursiveRemove(self):
    pool = svn.core.Pool()
    (wc, subpath) = gvn.wc.FindWorkingCopy(svn_path_internal_style(os.getcwd(),
                                                                   pool),
                                    cancel_func=None, notify_func=None,
                                    config=self.config, pool=pool)
    wc.Open(write_lock=True)
    testcommon.GetAuthBaton(self.config, pool, wc.project.repository)
    deltaman = gvn.wc.PostfixTextDeltaManager(wc)
    def action(path, pool):
      wc_path = wc.LocalPath(path)
      return gvn.wc.Edit(wc, wc_path, wc.Status([wc_path])[0][1], deltaman.Add)

    # Remove a directory and its children.
    path = 'greek-tree/A/D'
    testcommon.WCDelete(wc, path, pool)
    deltaman = gvn.wc.PostfixTextDeltaManager(wc)
    commit_info = gvn.commit.Drive(wc.project.repository, {'svn:log': ''},
                                   SVN_INVALID_REVNUM,
                                   [wc.RepoPath(path)], action,
                                   deltaman.Transmit, pool)
    wc.ProcessCommittedQueue(commit_info, pool)
    wc.Close()
    wc.Open(write_lock=False)
    # Is this right?
    self.assertEquals(wc.Entry(path, pool).revision, SVN_INVALID_REVNUM)
    wc.Close()

    # Note that you have to update the parent first before you can add
    # a new node of different kind.
    testcommon.RunSvn('update')

    # Add a file at path.
    testcommon.sleepf(open(path, 'w').write, 'hi\n')
    wc.Open(write_lock=True)
    testcommon.WCAdd(wc, path, pool)
    deltaman = gvn.wc.PostfixTextDeltaManager(wc)
    commit_info = gvn.commit.Drive(wc.project.repository, {'svn:log': ''},
                                   SVN_INVALID_REVNUM,
                                   [wc.RepoPath(path)], action,
                                   deltaman.Transmit, pool)
    wc.ProcessCommittedQueue(commit_info, pool)
    wc.Close()
    wc.Open(write_lock=False)
    self.assertEquals(wc.Entry(path, pool).kind, svn_node_file)
    self.assertEquals(wc.Entry(path, pool).revision, commit_info.revision)
    wc.Close()
